# mongodb数据库入门
搜索 nodejs 菜鸟教程

 https://www.runoob.com/nodejs/nodejs-mongodb.html

## (一) 安装数据库

### (1) 腾讯云安装

1. 登录宝塔面板 => 软件商店 => 搜索mongodb => 安装

2. 设置mongodb => 配置项 => BindIP：0.0.0.0

3. 使用robo3T连接数据库

   ![image-20220923142914316](bugImg/image-20220923142914316-1663914594036.png)





### (2) 本地安装mongodb

下载地址:  https://www.mongodb.com/download-center/community     版本选4.416

安装步骤:

https://m.runoob.com/mongodb/mongodb-window-install.html



## (二) 连接数据库

### (1) 普通模式

1. 安装依赖

   ```
   npm i mongodb
   ```

2. 根目录先建common/db.js

   ```JavaScript
   var MongoClient = require("mongodb").MongoClient;
   
   // 创建数据库对象
   async function createDb() {
     try {
       var url = "mongodb://localhost:27017/huruqing";
       var dbLink = await MongoClient.connect(url); 
       // dbo就是操作数据库的一个对象
       var dbo = dbLink.db("huruqing"); 
       console.log("数据库连接成功");
       return dbo; 
     } catch (error) {
       console.log("错误信息:", error);
     }
   }  
module.exports = createDb;
   ```

### (2) 单例模式

有数据库对象的时候就继续使用, 没有的话就创建一个

```JavaScript
var MongoClient = require("mongodb").MongoClient;

// 创建数据库对象
var dbo;
async function createDb() {
  try {
    var url = "mongodb://localhost:27017/huruqing";
    // 数据库连接
    var dbLink = await MongoClient.connect(url);
    if (dbo) {
      console.log('使用原来的数据库对象')
      return dbo;
    } else {
      // dbo就是操作数据库的一个对象
      dbo = dbLink.db("huruqing");
      console.log("数据库连接成功");
      return dbo;
    }
  } catch (error) {
    console.log("错误信息:", error);
  }
}
module.exports = createDb;
```



## (三) 添加数据

#### 例1. 添加数据接口

router.js

```JavaScript
// 导入创建数据库的方法
const createDb = require('../common/db.js');

router.all("/add", async function (ctx) {
  // 获取参数 
  var myobj = ctx.request.body;   
  try { 
    var dbase = await createDb();  
    var res = await dbase.collection("user").insertOne(myobj);
    ctx.body = {
      code: 1,
      msg: '插入成功',
      data:res
    };
  } catch (error) {
    ctx.body = {
      code: 0,
      msg: '插入失败',
    };
  } 
});
```

add.html

```html
axios.post(url, data).then(function (res) { 
	if (res.data.code == 1) {
		alert('添加成功');
		location.href = './index.html';
	} else {
		alert('添加失败');
	} 
})
```



#### 例2. 手动添加

直接在robo3T添加多条数据

```
db.getCollection('film').insert(
[
  {
    name: 我和我的父辈1,
    poster:
      "https://pic.maizuo.com/usr/movie/6d31257bd0c897f0a953b7097853d627.jpg",
    director: "吴京|章子怡|徐峥|沈腾",
    nation: "中国大陆",
    grade: 7.0
  },
  {
    name: 我和我的父辈2,
    poster:
      "https://pic.maizuo.com/usr/movie/6d31257bd0c897f0a953b7097853d627.jpg",
    director: "吴京|章子怡|徐峥|沈腾",
    nation: "中国大陆",
    grade: 8.0
  },
  {
    name: 我和我的父辈3,
    poster:
      "https://pic.maizuo.com/usr/movie/6d31257bd0c897f0a953b7097853d627.jpg",
    director: "吴京|章子怡|徐峥|沈腾",
    nation: "中国大陆",
    grade: 9.0
  }
]
)
```

按ctrl+enter就可以插入多条数据



## (四) 查询列表

router.js

```JavaScript
// 导入创建数据库的方法
const createDb = require('../common/db.js');
// 列表接口
router.all("/list", async function (ctx) {
  try {
    var dbase = await createDb();    
    let res = await dbase.collection("user"). find({}).toArray();
    ctx.body = {
      code:1,
      msg: 'success',
      data: res
    };
  } catch (error) {
    ctx.body = {
      code: 0,
      msg: 'error'
    }
  }
});
```

index.html

```javascript
  function getList() {
      var url = 'http://localhost:3000/list';
      axios.get(url).then(function (res) {
        if (res.data.code == 1) {
          list = res.data.data; 
          render(list);
        } else {
          alert('获取列表失败')
        } 
      })
    }
```

## (五) 删除数据

**前端**

```html
delFilm(_id, index) { 
    utils.post('/film/del', { _id }).then(res=> {
    	// 根据下标删除数组对应的成员
    	this.data.filmList.splice(index,1);
    	// 重新渲染页面
    this.rendList();
    }).catch(err=> {
    	console.log(err);
    }) 
},
```

**后端**

filmRouter.js

```javascript
router.all("/film/del", async (ctx) => {
  // 获取数据库对象
  const dbase = await createDb();
  console.log(ctx.params);
  // 结构, 从 ctx.params获取_id
  let { _id } = ctx.params;
  // whereStr, 删除的条件, ObjectId(_id)意思是把_id变成mongodb里的_id
  var whereStr = { _id: ObjectId(_id) };
  // 调用删除方法
  let res = await dbase.collection("film").deleteOne(whereStr); 
  if (res.deletedCount > 0) {
    ctx.body = {
      code: 666,
      msg: "删除成功",
      data: {
        _id
      }
    };
  } else {
    ctx.body = {
      code: 500,
      msg: "删除失败",
      data: {
        _id
      }
    };
  }
});
```



## (六) 扩展知识

### (1) koa设置cookie

```JavaScript
ctx.cookies.set('userId', 'xasdfasdfasd234234234');
```

### (2) 分页

limit()   返回给前端的数量

skip()  跳过的数量

```JavaScript
router.all("/film/list", async (ctx) => {
  let { currPage=1, pageSize=2 } = ctx.params; 

  const dbase = await createDb(); 
  // 分页
  let skipNum = (currPage-1)*pageSize;
  let res = await dbase.collection("film").find({}).limit(pageSize*1).skip(skipNum).toArray(); 

  ctx.body = {
    code: 666,
    msg: "sucess",
    data: res
  };
});
```

### (3) 排序

```
{ type: 1 }  // 按 type 字段升序
{ type: -1 } // 按 type 字段降序
```

### (4) id的问题

用uuid来生成唯一的id

1. `npm install uuid --save`

2. 使用

   ```
   let filmId = uuidv4();
   ctx.params.filmId = filmId;
   await dbase.collection("film").insertOne(ctx.params);
   ```

### (5) 日期问题

一般添加数据的时候会加上创建时间和更新时间

```JavaScript
  let filmId = uuidv4();
  let cTime = Date.now();
  let uTime = Date.now();
  ctx.params = {
    ...ctx.params,
    filmId,
    cTime,
    uTime
  } 
```











